import streamlit as st

try:
    from keywords import check_keyword
    from keywords import TODO_KEYWORD, TODO_COMPLETED, TODO_INCOMPLETE, TODO_SUCCESS, SUCCESS
except:
    from .keywords import check_keyword
    from .keywords import TODO_KEYWORD, TODO_COMPLETED, TODO_INCOMPLETE, TODO_SUCCESS, SUCCESS

def to_do_list(tasks, header="", success=TODO_SUCCESS):
    """ Renders the tasks as a to-do list, with optional header and success message.
    The tasks are a dictionary of tasks (supposed to be ordered as Python +3.6) 
    with their completed (True) or to-do (False) status as a checkbox.

    :param tasks: dictionary of tasks in format string:bool
    :type tasks: dict
    :param header: description of the tasks
    :type header: str, optional
    :param success: success message
    :type success: str, optional
    :return: boolean with the exit status of the function
    :rtype: bool
    """
    if len(tasks)==0:
        st.error("There are no tasks to display.")
    cb_list = []
    st.markdown(header)
    for task, status in tasks.items():
        key = (header + task).lower().replace(" ", "_")
        cb = st.checkbox(task, value=status, key=key)
        cb_list.append(cb)
    if len(cb_list)>0 and all(cb for cb in cb_list):
        st.balloons()
        st.success(success)
        return True
    else:
        return False


def to_do_list_parser(lines):
    """Parses a list of lines into a dictionary with the parsed values.

    :param lines: list of lines
    :type lines: list
    :return: parsed values for tasks, and optionally the header and success message.
    :rtype: dict
    """
    # Dict to store the parsed values
    parse_dict = {
                    "tasks":{},
                    "header":"",
                    "success":TODO_SUCCESS,
                }
    for i, line in enumerate(lines):
        if i==0:
            if check_keyword(line, TODO_KEYWORD):
                continue
            else:
                break
        elif line.startswith(TODO_COMPLETED):
            task = line[len(TODO_COMPLETED):].strip()
            parse_dict["tasks"][task] = True
        elif line.startswith(TODO_INCOMPLETE):
            task = line[len(TODO_INCOMPLETE):].strip()
            parse_dict["tasks"][task] = False
        elif line.startswith(SUCCESS):
            parse_dict["success"] = line[len(SUCCESS):].strip()
        else:
            parse_dict["header"] += line
    return parse_dict


def to_do_list_from_lines(lines):
    """Renders a to-do list from a list of lines.

    :param lines: list of lines
    :type lines: list
    :return: None
    """
    parse_dict = to_do_list_parser(lines)
    to_do_list(tasks=parse_dict["tasks"], 
                header=parse_dict["header"], 
                success=parse_dict["success"])
    return


from datetime import datetime
import streamlit as st
import subprocess
import random
import os

try:
    from keywords import ADMIN_PASSWORD
except: 
    from .keywords import ADMIN_PASSWORD

def password_entered():
    """
    Checks whether a password entered by the user is correct.
    Checks agains secrets (if defined) otherwise checks against hardcoded password.
    """
    # Get the password
    try:
        PASSWORD = st.secrets["ADMIN_PASSWORD"]
    except:
        PASSWORD = ADMIN_PASSWORD
    # Compare against password entered by the user
    if st.session_state["password"] == PASSWORD:
        st.session_state["password_correct"] = True
        del st.session_state["password"]  # don't store password
    else:
        st.session_state["password_correct"] = False
    # Delete password from session state
    del PASSWORD
    return

def password_login(user_id):
    if "password_correct" not in st.session_state:
        # First run, show input for password.
        st.text_input("Password", type="password", on_change=password_entered, key="password")
    elif not st.session_state["password_correct"]:
        # Password not correct, show input + error.
        st.text_input("Password", type="password", on_change=password_entered, key="password")
        st.error("😕 Password incorrect")

def get_datetime_string():
    # datetime object containing current date and time
    dt_string = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    return dt_string

def get_git_revision_short_hash() -> str:
    """"
    Improved upon version found at 
    https://stackoverflow.com/questions/14989858/get-the-current-git-hash-in-a-python-script/21901260#21901260"
    """
    try:
        return subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD']).decode('ascii').strip()
    except:
        return "not_git"

def download_file(df, button_label, filename, where=None):
    """
    """
    data_csv = df.to_csv().encode('utf-8')
    if where:
        btn = where.download_button(
                label=button_label,
                data=data_csv,
                file_name=filename,
            )
    else:
        btn = st.download_button(
                label=button_label,
                data=data_csv,
                file_name=filename,
            )
    return

def get_token():
    t1 = get_random_string(3, "letters")
    t2 = get_random_string(3, "numbers")
    return t1 + t2

def get_random_string(length, elements):
    if elements == "letters":
        elements = "abcdefghkmnopqrstxyz" # no confusing characters
    elif elements == "numbers":
        elements = "0123456789"
    else:
        elements = string.digits + string.ascii_letters
    # With combination of lower and upper case
    result_str = ''.join(random.choice(elements) for i in range(length))
    # print random string
    return result_str


import streamlit as st
import os
import pandas as pd
import altair as alt
try:
    from keywords import USERS_FILENAME, ANSWER_FILENAME
    from helpers import password_login, download_file, get_git_revision_short_hash
except: 
    from .keywords import USERS_FILENAME, ANSWER_FILENAME, ADMIN_PASSWORD
    from .helpers import password_login, download_file, get_git_revision_short_hash

def admin_page():
    """"
    Shows a password page to the admin.
    """
    if "password_correct" not in st.session_state or not st.session_state["password_correct"]:
        admin_login_page()
    else:
        c1, c2 = st.columns(2)
        c1.title("Admin page")
        # Show configuration
        options = ["Configuration", "User info", "Answer info"]
        option = c2.selectbox("Choose an option", options)
        # Show users
        if option == options[0]:
            configuration()
        if option == options[1]:
            user_info()
        if option == options[2]:
            answers_info()
    return

def configuration():
    """
    Shows available info on users.
    """
    st.header("Configuration")
    c1, c2, c3 = st.columns(3)
    c1.metric(label="Save answers?", value=f"{st.session_state.save_answers}")
    commit_hash = get_git_revision_short_hash()
    c2.metric(label="Current commit", value=f"{commit_hash}")


def user_info():
    """
    Shows available info on users.
    """
    if os.path.exists(USERS_FILENAME):
        # If file exists, read the file
        df = pd.read_csv(USERS_FILENAME)
        # Provide some basic stats
        n_users = len(df)
        first_user = df.datetime.min()
        last_user = df.datetime.max()
        c1, c2, c3 = st.columns(3)
        c1.metric(label="# users", value=f"{n_users}")
        c2.metric(label="First user created on", value=f"{first_user}")
        c3.metric(label="Last user created on", value=f"{last_user}")
        # Plot the users by day
        df['day'] = pd.to_datetime(df['datetime'], format="%Y-%m-%d %H:%M:%S").dt.strftime('%Y-%m-%d')
        alt_df = df.groupby("day").count().rename(columns={"user_id": "# users"}).reset_index()
        alt_plot = alt.Chart(alt_df).mark_line(point=alt.OverlayMarkDef(color="blue")).encode(x="day", y="# users")
        #st.write(alt_df)
        st.altair_chart(alt_plot)
        # Allow to download the file
        download_file(df, button_label="Download user data", filename="users.csv")
        # Show the file or a graph
        #st.write(df)
    else:
        st.warning("No users file found.")    

def answers_info():
    """
    Shows available info on answers.
    """
    # Show answers
    if os.path.exists(ANSWER_FILENAME):
        df = pd.read_csv(ANSWER_FILENAME)
        # Provide some basic stats
        n_answers = len(df)
        n_unique_users = df["user_id"].nunique()
        n_unique_questions = df["question"].nunique()
        correctness = df["correct?"].mean()
        c1, c2, c3, c4 = st.columns(4)
        c1.metric(label="# answers", value=f"{n_answers}")
        c2.metric(label="# users on answers", value=f"{n_unique_users}")
        c3.metric(label="# questions", value=f"{n_unique_questions}")
        c4.metric(label="Average correct?", value=f"{100*correctness}%")
        # Plot the users by day
        df['day'] = pd.to_datetime(df['datetime'], format="%Y-%m-%d %H:%M:%S").dt.strftime('%Y-%m-%d')
        alt_df = df.groupby("day").count().rename(columns={"user_id": "# users"}).reset_index()
        alt_plot = alt.Chart(alt_df).mark_line(point=alt.OverlayMarkDef(color="blue")).encode(x="day", y="# users")
        #st.write(alt_df)
        st.altair_chart(alt_plot)
        # Allow to download the file
        c1, c2 = st.columns(2)
        df_last_answer = df.drop_duplicates(keep="last")
        download_file(df, "Download all answers", "all_answers.csv", where=c1)
        download_file(df_last_answer, "Download last answers", "last_answers.csv", where=c2)
        # Show the file or a graph
        #st.write(df)
    else:
        st.warning("No answers file found.")

def admin_login_page():
    """"
    Shows a password page to the admin.
    """
    st.title("Admin Login")
    st.write("Please enter the password to access the admin page.")
    password_login("admin")


import streamlit as st

try:
    from keywords import check_keyword
    from keywords import SINGLE_CHOICE_KEYWORD, SINGLE_CHOICE_CORRECT, SINGLE_CHOICE_WRONG
    from keywords import BUTTON, SUCCESS, ERROR, DEFAULT_SUCCESS_MESSAGE, DEFAULT_ERROR_MESSAGE, DEFAULT_BUTTON_MESSAGE
    from answers import save_answer
except:
    from .keywords import check_keyword
    from .keywords import SINGLE_CHOICE_KEYWORD, SINGLE_CHOICE_CORRECT, SINGLE_CHOICE_WRONG, BUTTON, SUCCESS, ERROR
    from .keywords import BUTTON, SUCCESS, ERROR, DEFAULT_SUCCESS_MESSAGE, DEFAULT_ERROR_MESSAGE, DEFAULT_BUTTON_MESSAGE
    from .answers import save_answer


def single_choice(question, options, answer_index,
                    success=DEFAULT_SUCCESS_MESSAGE, 
                    error=DEFAULT_ERROR_MESSAGE, 
                    button=DEFAULT_BUTTON_MESSAGE):
    """Renders a single-choice question from arguments.

    :param question: question to be displayed before the single-choice options
    :type question: str
    :param options: list of options to be displayed.
    :type options: str
    :param answer: index (starting at 0) of the expected answer to the question
    :type answer: int
    :param success: message to be displayed when the user answers correctly
    :type success: str, optional
    :param error: message to be displayed when the user answers incorrectly
    :type error: str, optional
    :param button: message to be displayed on the button that checks the answer
    :type button: str, optional
    :return: tuple of booleans with button press status and correctness of answer
    :rtype: tuple of bool
    """
    if len(question)==0:
        st.error("Please provide a question")
        return None, None
    elif len(options)<2:
        st.error("Must have at least 2 options")
        return None, None
    else:
        # Write the options with radio button: only one option can be selected
        user_answer = st.radio(question, options=options)
        # Check if correct answer
        key = ("single-choice:" + question + "".join(options)).lower().replace(" ", "_")
        if st.button(button, key=key):
            # Check ansers
            if options.index(user_answer) == answer_index:
                if success:
                    st.success(success)
                is_correct = True
            else:
                if error:
                    st.error(error)
                is_correct = False
            # Save the answers, if required
            if "save_answers" in st.session_state and st.session_state.save_answers:
                save_answer(question, is_correct=is_correct, user_answer=user_answer, correct_answer=options[answer_index])
            # Return if button is pressed and answer evaluation
            return True, is_correct
        else:
            # Return if button is not pressed
            return False, None

def single_choice_parser(lines):
    """Parses a list of lines into a dictionary with the parsed values.

    :param lines: list of lines
    :type lines: list
    :return: parsed values for the single choice quizz type.
    :rtype: dict
    """
    # Define default feedback messages 
    parse_dict = {
                    "question":"",
                    "options":[],
                }
    answer = ""
    # Iterate through lines and process each line accordingly
    for i, line in enumerate(lines):
        if i==0:
            if check_keyword(line, SINGLE_CHOICE_KEYWORD):
                continue
            else:
                break
        elif line.startswith(SINGLE_CHOICE_CORRECT):
            option_text = line[len(SINGLE_CHOICE_CORRECT):].strip()
            parse_dict["options"].append(option_text)
            answer = option_text
        elif line.startswith(SINGLE_CHOICE_WRONG):
            option_text = line[len(SINGLE_CHOICE_WRONG):].strip()
            parse_dict["options"].append(option_text)
        elif line.startswith(BUTTON):
            msg = line[len(BUTTON):].strip()
            parse_dict["button"] = msg  
        elif line.startswith(SUCCESS):
            msg = line[len(SUCCESS):].strip()
            parse_dict["success"] = msg  
        elif line.startswith(ERROR):
            msg = line[len(ERROR):].strip()
            parse_dict["error"] = msg
        else:
            parse_dict["question"] += line + "\n"
    # Store the answer_index
    parse_dict["answer_index"] = parse_dict["options"].index(answer)
    return parse_dict



def single_choice_from_lines(lines):
    """Renders a single choice question from a list of lines.

    :param lines: list of lines
    :type lines: list
    :return: None
    """
    parse_dict = single_choice_parser(lines)
    single_choice(**parse_dict)
    return




import streamlit as st
from glob import glob
import os
import pandas as pd

try:
    from .file_reader import get_all_files, create_buttons
    from .login import create_new_user_and_token, get_user_from_token, get_token_from_user  
    from .admin import admin_page
    from .helpers import get_git_revision_short_hash
    from .render import render_file
except Exception as e:
    print("Cannot import from . ", e)    
    from file_reader import get_all_files, create_buttons
    from login import create_new_user_and_token, get_user_from_token, get_token_from_user
    from admin import admin_page
    from helpers import get_git_revision_short_hash
    from render import render_file
except Exception as e:
    print("Cannot import! ", e)    

def set_chapter_config(path="pages",
                    toc=False,
                    button="top",
                    button_previous="⬅️",
                    button_next="➡️",
                    button_refresh="🔄",
                    on_load_header=None,
                    on_load_footer=None,
                    save_answers=False,
                    ):
    """Sets the book configuration, and displays the selected file.

    :param path: The path to root directory of the the files (py or md) to be rendered as pages of the book. 
    :type path: string, dict
    :param toc: If True, it will display the table of contents for the files on the path. Defaults to False.
    :type toc: bool
    :param button: "top" (default behavior) or "bottom".
    :type button: str
    :param button_previous: icon or text for the previous button.
    :type button_previous: str
    :param button_next: icon or text for the next button.
    :type button_next: str
    :param button_refresh: icon or text for the refresh button.
    :type button_refresh: str
    :param on_load_header: function to be called before the page is loaded.
    :type on_load_header: function
    :param on_load_footer: function to be called after the page is loaded.
    :type on_load_footer: function
    :param save_answers: If True, it will save the answers in a csv file. Defaults to False.
    :type save_answers: bool
    :return: None
    """
    # Observation: File number goes from 0 to n-1.

    # Quick and dirty
    path = str(path)

    # Store the parameters
    if "save_answers" not in st.session_state:
        st.session_state.save_answers = save_answers
    if "commit_hash" not in st.session_state:
        st.session_state.commit_hash =  get_git_revision_short_hash()

    # Admin Login: if requested by the user on the url, it redirects to a specific amin page
    # Never shows the content, it will stay on the admin page.
    query_params = st.experimental_get_query_params()
    if "user" in query_params and "admin" in query_params["user"]:
        # Here we handle everything related to the admin
        admin_page()
        return # Don't show anything else!

    # User login
    query_params = st.experimental_get_query_params()
    if "token" in query_params:
        # Here we must handle the user session
        token = query_params["token"][0] #Just consider the first one
        ## If the token is wrong, show a special page (only)
        user_id = get_user_from_token(token)
        st.session_state.user_id = user_id
        if user_id is None:
            # Wrong token
            st.markdown("Wrong token!")
            user_id, token = create_new_user_and_token()
            query_parameters = {"token": token}
            def on_click():
                st.experimental_set_query_params(**query_parameters)
            if st.button("Go back to the main page?", on_click=on_click):
                st.warning("You will be redirected to the main page.")
            return
    else:
        # token not in query params
        if "user_id" not in st.session_state:
            # No user logged in, get user_id and token to redirect
            user_id, token = create_new_user_and_token()
            st.session_state.user_id = user_id
        else:
            # We know the user_id, so we can get the token and reroute
            token = get_token_from_user(st.session_state.user_id)
        query_parameters = {"token": token}
        st.experimental_set_query_params(**query_parameters)

    # Save and answers behavior
    if st.session_state.save_answers:
        if "warned_about_save_answers" not in st.session_state:
            def on_click():
                st.session_state.warned_about_save_answers = True
            c1, c2 = st.columns([8,1])
            c1.warning("Your answers will be saved.\n\nYou can relaunch the app using the custom url.")
            c2.markdown("\n\n")
            c2.markdown("\n\n")
            c2.button("Dismiss", on_click=on_click)
            
    # Get the files at path level (only files, not folders)
    file_list = get_all_files(path)

    # Check that we have at least 1 file to render
    if len(file_list) == 0:
        st.error(f"No files were found at the given path. Please check the provided path: {path}")
        return

    # Initialize the session state variables
    if "toc" not in st.session_state:
        st.session_state.toc = toc

    # Save button configuration
    if "button" not in st.session_state:
        st.session_state.button = button

    # Page number
    if "page_number" not in st.session_state:
        st.session_state.page_number = 0

    # Update file_fullpath
    selected_file_fullpath = file_list[st.session_state.page_number]
    caption_text = f"Page {st.session_state.page_number+1} of {st.session_state.total_files}. File: {selected_file_fullpath}"

    if st.session_state.toc:
        option = st.radio("Table of contents", options=file_list)
        st.session_state.page_number = file_list.index(option)
        st.button("Go to page", on_click=on_toc_table_click, key="gotopage")
    else:
        # Execute the on_load_header function
        if on_load_header:
            on_load_header()

        # If required, put the button on top of the page. Use columns for alignment
        if st.session_state.button in ["top", "both"]:
            create_buttons(caption_text, 
                            button_previous, button_next, button_refresh,
                            key="top")

        # Render the file using the magic
        try:
            render_file(selected_file_fullpath)
        except Exception as e:
            st.exception(e)

        # If required, put the button on the bottom of the page. Use columns for alignment
        if st.session_state.button in ["bottom", "both"]:
            create_buttons(caption_text, button_previous, button_next, button_refresh, 
                            key="bottom")

        # Execute the on_load_footer function
        if on_load_footer:
            on_load_footer()
    return

def set_book_config(options, paths,
                        menu_title="Select a chapter",
                        menu_icon="book", 
                        icons=None, 
                        orientation=None, 
                        styles=None,
                        save_answers=False
                        ):
    """Creates a book using the streamlit_option_menu library.
    Renders each of the corresponding chapters based on their properties.
    Uses the same configurations used by `streamlit-option-menu 
    <https://github.com/victoryhb/streamlit-option-menu>`_
    and icons from `bootstrap-icons <https://icons.getbootstrap.com/>`_.

    :param options: List of chapter names to be displayed
    :type options: list of str
    :param paths: List of chapter paths containging the pages (py, md) to be displayed
    :type paths: list of str
    :param menu_title: Title of the menu, can be empty to be skipped.
    :type menu_title: str
    :param menu_icon: Icon to be used on the menu, from bootstrap icons.
    :type menu_icon: str
    :param icons: Icons to be used. Can be a single one used for all books, or a list of icons for each book.
    :type menu_icon: str or list of str
    :param orientation: Orientation of the menu. Can be "horizontal" or "vertical".
    :type orientation: str
    :param styles: Styles to be used. See the documentation of streamlit_option_menu.
    :type styles: dict
    :param save_answers: If True, it will save the answers in a csv file. Defaults to False.
    :type save_answers: bool
    :return: None
    """
    from streamlit_option_menu import option_menu

    # Initialize variables
    if "page_number" not in st.session_state:
        st.session_state.page_number = 0
    if "book_number" not in st.session_state:
        st.session_state.book_number = 0

    # Pack the arguments
    execution_dict = {
                        "menu_title": menu_title, 
                        "options": options,
                    }
    args = [menu_icon, icons, orientation, styles]
    if menu_icon is not None: execution_dict["menu_icon"] = menu_icon
    if icons is not None: execution_dict["icons"] = icons
    if orientation is not None: execution_dict["orientation"] = orientation
    if styles is not None: execution_dict["styles"] = styles

    # Execute the menu
    if orientation=="horizontal":
        selected_book_name = option_menu(**execution_dict)
    else:
        with st.sidebar:
            selected_book_name = option_menu(**execution_dict)
    
    # Update variables
    selected_book_number = options.index(selected_book_name)
    if st.session_state.book_number != selected_book_number:
        st.session_state.book_number = selected_book_number
        st.session_state.page_number = 0
        
    # Launch the corresponding chapter
    set_chapter_config(path=paths[selected_book_number], save_answers=save_answers)



import os
import pandas as pd
import streamlit as st

try:
    from helpers import get_datetime_string, download_file, get_token
    from keywords import USERS_FILENAME, ANSWER_FILENAME
except: 
    from .helpers import get_datetime_string, download_file, get_token
    from .keywords import USERS_FILENAME, ANSWER_FILENAME

def create_user_df(user_id, token):
    """
    Create a dataframe with the user information.
    """
    dt_string = get_datetime_string()
    user_df = pd.DataFrame(columns=["user_id", "token", "datetime"], data=[[user_id, token, dt_string]])
    return user_df

def get_user_from_token(query_token):
    df = pd.read_csv(USERS_FILENAME)
    m = df["token"] == query_token
    if m.any():
        user_for_token = df.user_id[m].values[0] # the first (and only) one
        return user_for_token
    else:
        return None

def create_new_user_and_token():
    """
    Returns the user id and token.
    """
    # Create folder, if not there
    try:
        os.mkdir("./tmp/")
    except:
        pass
    # Check for user file, create if not there
    if not os.path.exists("./tmp/users.csv"):
        df0 = create_user_df(0,"abc123")
        df0.to_csv(USERS_FILENAME, index=False)
    # Take a new user
    df_users = pd.read_csv(USERS_FILENAME)
    user_id = df_users["user_id"].max() + 1
    user_token = get_token()
    df_new_user = create_user_df(user_id, user_token)
    df_users = pd.concat([df_users, df_new_user])
    df_users.to_csv(USERS_FILENAME, index=False)
    # Create the token
    return (user_id, user_token)

def get_token_from_user(user_id):
    df = pd.read_csv(USERS_FILENAME)
    m = df["user_id"] == user_id
    if m.any():
        token = df.token[m].values[0]
        return token
    else:
        return None


import streamlit as st
import os

try:
    from helpers import get_datetime_string, get_git_revision_short_hash
    from keywords import ANSWER_FILENAME 
except: 
    from .helpers import get_datetime_string, get_git_revision_short_hash
    from .keywords import ANSWER_FILENAME 

def create_answer_file():
    if not os.path.exists(ANSWER_FILENAME):
        with open(ANSWER_FILENAME, "w") as f:
            f.write("commit_hash,datetime,user_id,question,correct?,user_answer,correct_answer\n")
        return

def save_answer(question, is_correct, user_answer, correct_answer):
    """
    Save the answer to the question.
    """
    # Parameters
    create_answer_file()
    commit_hash = st.session_state.commit_hash
    dt_string = get_datetime_string()
    user_id = st.session_state.user_id
    # Clean question and answers
    question = question.replace("\n", "\\n")
    user_answer = str(user_answer).replace("\n", "\\n")
    correct_answer = str(correct_answer).replace("\n", "\\n")
    with open(ANSWER_FILENAME, "a") as f:
        f.write(f'"{commit_hash}","{dt_string}","{user_id}","{question}","{is_correct}","{user_answer}","{correct_answer}"\n')   
    return


from streamlit.components.v1 import html

try:
    from keywords import SHARE_KEYWORD 
    from keywords import check_keyword
except:
    from .keywords import SHARE_KEYWORD 
    from .keywords import check_keyword

def share_parser(lines):
    """Parses a list of lines into a dictionary with the parsed values.

    :param lines: list of lines
    :type lines: list
    :return: parsed values for text and url
    :rtype: dict
    """
    # Dict to store the parsed values
    parse_dict = {
                    "my_text":"",
                    "my_url":"",
                }
    for i, line in enumerate(lines):
        if i==0:
            if check_keyword(line, SHARE_KEYWORD):
                continue
            else:
                break
        elif i==1:
            parse_dict["my_text"] = line.strip()
        elif i==2:
            parse_dict["my_url"] = line.strip()
    return parse_dict

def share_from_lines(lines):
    """Renders the share buttons from a list of lines.

    :param lines: list of lines
    :type lines: list
    :return: None
    """
    parse_dict = share_parser(lines)
    share(my_text=parse_dict["my_text"], 
            my_url=parse_dict["my_url"], )
    return

def share(my_text, my_url):
    """
    This function takes a url and a text and displays
    clickable sharing buttons in html.

    :param my_text: the text to share on social media
    :type my_text: str
    :param my_url: the url to share on social media
    :type my_url: str
    """

    # Define the css style for the sharing buttons
    my_css = """
    .resp-sharing-button__link,
    .resp-sharing-button__icon {
        display: inline-block
    }

    .resp-sharing-button__link {
        text-decoration: none;
        color: #fff;
        margin: 0.5em
    }

    .resp-sharing-button {
        border-radius: 5px;
        transition: 25ms ease-out;
        padding: 0.5em 0.75em;
        font-family: Helvetica Neue,Helvetica,Arial,sans-serif
    }

    .resp-sharing-button__icon svg {
        width: 1em;
        height: 1em;
        margin-right: 0.4em;
        vertical-align: top
    }

    .resp-sharing-button--small svg {
        margin: 0;
        vertical-align: middle
    }

    /* Non solid icons get a stroke */
    .resp-sharing-button__icon {
        stroke: #fff;
        fill: none
    }

    /* Solid icons get a fill */
        .resp-sharing-button__icon--solid,
        .resp-sharing-button__icon--solidcircle {
        fill: #fff;
        stroke: none
    }

    .resp-sharing-button--twitter {
        background-color: #55acee
    }

    .resp-sharing-button--twitter:hover {
        background-color: #2795e9
    }

    .resp-sharing-button--pinterest {
        background-color: #bd081c
    }

    .resp-sharing-button--pinterest:hover {
        background-color: #8c0615
    }

    .resp-sharing-button--facebook {
        background-color: #3b5998
    }

    .resp-sharing-button--facebook:hover {
        background-color: #2d4373
    }

    .resp-sharing-button--tumblr {
        background-color: #35465C
    }

    .resp-sharing-button--tumblr:hover {
        background-color: #222d3c
    }

    .resp-sharing-button--reddit {
        background-color: #5f99cf
    }

    .resp-sharing-button--reddit:hover {
        background-color: #3a80c1
    }

    .resp-sharing-button--google {
        background-color: #dd4b39
    }

    .resp-sharing-button--google:hover {
        background-color: #c23321
    }

    .resp-sharing-button--linkedin {
        background-color: #0077b5
    }

    .resp-sharing-button--linkedin:hover {
        background-color: #046293
    }

    .resp-sharing-button--email {
        background-color: #777
    }

    .resp-sharing-button--email:hover {
        background-color: #5e5e5e
    }

    .resp-sharing-button--xing {
        background-color: #1a7576
    }

    .resp-sharing-button--xing:hover {
        background-color: #114c4c
    }

    .resp-sharing-button--whatsapp {
        background-color: #25D366
    }

    .resp-sharing-button--whatsapp:hover {
        background-color: #1da851
    }

    .resp-sharing-button--hackernews {
        background-color: #FF6600
    }
    .resp-sharing-button--hackernews:hover, .resp-sharing-button--hackernews:focus {   background-color: #FB6200 }

    .resp-sharing-button--vk {
        background-color: #507299
    }

    .resp-sharing-button--vk:hover {
        background-color: #43648c
    }

    .resp-sharing-button--facebook {
        background-color: #3b5998;
        border-color: #3b5998;
    }

    .resp-sharing-button--facebook:hover,
    .resp-sharing-button--facebook:active {
        background-color: #2d4373;
        border-color: #2d4373;
    }

    .resp-sharing-button--twitter {
        background-color: #55acee;
        border-color: #55acee;
    }

    .resp-sharing-button--twitter:hover,
    .resp-sharing-button--twitter:active {
        background-color: #2795e9;
        border-color: #2795e9;
    }

    .resp-sharing-button--email {
        background-color: #777777;
        border-color: #777777;
    }

    .resp-sharing-button--email:hover,
    .resp-sharing-button--email:active {
        background-color: #5e5e5e;
        border-color: #5e5e5e;
    }

    .resp-sharing-button--linkedin {
        background-color: #0077b5;
        border-color: #0077b5;
    }

    .resp-sharing-button--linkedin:hover,
    .resp-sharing-button--linkedin:active {
        background-color: #046293;
        border-color: #046293;
    }

    .resp-sharing-button--whatsapp {
        background-color: #25D366;
        border-color: #25D366;
    }

    .resp-sharing-button--whatsapp:hover,
    .resp-sharing-button--whatsapp:active {
        background-color: #1DA851;
        border-color: #1DA851;
    }

    .resp-sharing-button--telegram {
        background-color: #54A9EB;
    }

    .resp-sharing-button--telegram:hover {
        background-color: #4B97D1;}
    """

    # Define the html
    my_html = f"""
    <!-- Sharingbutton Facebook -->
    <span>
        <a class="resp-sharing-button__link" href="https://facebook.com/sharer/sharer.php?u={my_url}" target="_blank" rel="noopener" aria-label="">
        <div class="resp-sharing-button resp-sharing-button--facebook resp-sharing-button--small"><div aria-hidden="true" class="resp-sharing-button__icon resp-sharing-button__icon--solidcircle">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M12 0C5.38 0 0 5.38 0 12s5.38 12 12 12 12-5.38 12-12S18.62 0 12 0zm3.6 11.5h-2.1v7h-3v-7h-2v-2h2V8.34c0-1.1.35-2.82 2.65-2.82h2.35v2.3h-1.4c-.25 0-.6.13-.6.66V9.5h2.34l-.24 2z"/></svg>
            </div>
        </div>
        </a>

        <!-- Sharingbutton Twitter -->
        <a class="resp-sharing-button__link" href="https://twitter.com/intent/tweet/?text={my_text}&amp;url={my_url}" target="_blank" rel="noopener" aria-label="">
        <div class="resp-sharing-button resp-sharing-button--twitter resp-sharing-button--small"><div aria-hidden="true" class="resp-sharing-button__icon resp-sharing-button__icon--solidcircle">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M12 0C5.38 0 0 5.38 0 12s5.38 12 12 12 12-5.38 12-12S18.62 0 12 0zm5.26 9.38v.34c0 3.48-2.64 7.5-7.48 7.5-1.48 0-2.87-.44-4.03-1.2 1.37.17 2.77-.2 3.9-1.08-1.16-.02-2.13-.78-2.46-1.83.38.1.8.07 1.17-.03-1.2-.24-2.1-1.3-2.1-2.58v-.05c.35.2.75.32 1.18.33-.7-.47-1.17-1.28-1.17-2.2 0-.47.13-.92.36-1.3C7.94 8.85 9.88 9.9 12.06 10c-.04-.2-.06-.4-.06-.6 0-1.46 1.18-2.63 2.63-2.63.76 0 1.44.3 1.92.82.6-.12 1.95-.27 1.95-.27-.35.53-.72 1.66-1.24 2.04z"/></svg>
            </div>
        </div>
        </a>

        <!-- Sharingbutton E-Mail -->
        <a class="resp-sharing-button__link" href="mailto:?subject={my_text}&amp;body={my_url}" target="_self" rel="noopener" aria-label="">
        <div class="resp-sharing-button resp-sharing-button--email resp-sharing-button--small"><div aria-hidden="true" class="resp-sharing-button__icon resp-sharing-button__icon--solidcircle">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M12 0C5.38 0 0 5.38 0 12s5.38 12 12 12 12-5.38 12-12S18.62 0 12 0zm8 16c0 1.1-.9 2-2 2H6c-1.1 0-2-.9-2-2V8c0-1.1.9-2 2-2h12c1.1 0 2 .9 2 2v8z"/><path d="M17.9 8.18c-.2-.2-.5-.24-.72-.07L12 12.38 6.82 8.1c-.22-.16-.53-.13-.7.08s-.15.53.06.7l3.62 2.97-3.57 2.23c-.23.14-.3.45-.15.7.1.14.25.22.42.22.1 0 .18-.02.27-.08l3.85-2.4 1.06.87c.1.04.2.1.32.1s.23-.06.32-.1l1.06-.9 3.86 2.4c.08.06.17.1.26.1.17 0 .33-.1.42-.25.15-.24.08-.55-.15-.7l-3.57-2.22 3.62-2.96c.2-.2.24-.5.07-.72z"/></svg>
            </div>
        </div>
        </a>

        <!-- Sharingbutton LinkedIn -->
        <a class="resp-sharing-button__link" href="https://www.linkedin.com/shareArticle?mini=true&amp;url={my_url}&amp;title={my_text}&amp;summary={my_text}&amp;source={my_url}" target="_blank" rel="noopener" aria-label="">
        <div class="resp-sharing-button resp-sharing-button--linkedin resp-sharing-button--small"><div aria-hidden="true" class="resp-sharing-button__icon resp-sharing-button__icon--solidcircle">
            <svg version="1.1" x="0px" y="0px" width="24px" height="24px" viewBox="0 0 24 24" enable-background="new 0 0 24 24" xml:space="preserve">
                <path d="M12,0C5.383,0,0,5.383,0,12s5.383,12,12,12s12-5.383,12-12S18.617,0,12,0z M9.5,16.5h-2v-7h2V16.5z M8.5,7.5 c-0.553,0-1-0.448-1-1c0-0.552,0.447-1,1-1s1,0.448,1,1C9.5,7.052,9.053,7.5,8.5,7.5z M18.5,16.5h-3V13c0-0.277-0.225-0.5-0.5-0.5 c-0.276,0-0.5,0.223-0.5,0.5v3.5h-3c0,0,0.031-6.478,0-7h3v0.835c0,0,0.457-0.753,1.707-0.753c1.55,0,2.293,1.12,2.293,3.296V16.5z" />
            </svg>
            </div>
        </div>
        </a>

        <!-- Sharingbutton WhatsApp -->
        <a class="resp-sharing-button__link" href="https://wa.me/?text={my_text}%20{my_url}" target="_blank" rel="noopener" aria-label="">
        <div class="resp-sharing-button resp-sharing-button--whatsapp resp-sharing-button--small"><div aria-hidden="true" class="resp-sharing-button__icon resp-sharing-button__icon--solidcircle">
            <svg xmlns="http://www.w3.org/2000/svg" height="24" width="24" viewBox="0 0 24 24"><path d="m12 0c-6.6 0-12 5.4-12 12s5.4 12 12 12 12-5.4 12-12-5.4-12-12-12zm0 3.8c2.2 0 4.2 0.9 5.7 2.4 1.6 1.5 2.4 3.6 2.5 5.7 0 4.5-3.6 8.1-8.1 8.1-1.4 0-2.7-0.4-3.9-1l-4.4 1.1 1.2-4.2c-0.8-1.2-1.1-2.6-1.1-4 0-4.5 3.6-8.1 8.1-8.1zm0.1 1.5c-3.7 0-6.7 3-6.7 6.7 0 1.3 0.3 2.5 1 3.6l0.1 0.3-0.7 2.4 2.5-0.7 0.3 0.099c1 0.7 2.2 1 3.4 1 3.7 0 6.8-3 6.9-6.6 0-1.8-0.7-3.5-2-4.8s-3-2-4.8-2zm-3 2.9h0.4c0.2 0 0.4-0.099 0.5 0.3s0.5 1.5 0.6 1.7 0.1 0.2 0 0.3-0.1 0.2-0.2 0.3l-0.3 0.3c-0.1 0.1-0.2 0.2-0.1 0.4 0.2 0.2 0.6 0.9 1.2 1.4 0.7 0.7 1.4 0.9 1.6 1 0.2 0 0.3 0.001 0.4-0.099s0.5-0.6 0.6-0.8c0.2-0.2 0.3-0.2 0.5-0.1l1.4 0.7c0.2 0.1 0.3 0.2 0.5 0.3 0 0.1 0.1 0.5-0.099 1s-1 0.9-1.4 1c-0.3 0-0.8 0.001-1.3-0.099-0.3-0.1-0.7-0.2-1.2-0.4-2.1-0.9-3.4-3-3.5-3.1s-0.8-1.1-0.8-2.1c0-1 0.5-1.5 0.7-1.7s0.4-0.3 0.5-0.3z"/></svg>
            </div>
        </div>
        </a>

        <!-- Sharingbutton Telegram -->
        <a class="resp-sharing-button__link" href="https://telegram.me/share/url?text={my_text}&amp;url={my_url}" target="_blank" rel="noopener" aria-label="">
        <div class="resp-sharing-button resp-sharing-button--telegram resp-sharing-button--small"><div aria-hidden="true" class="resp-sharing-button__icon resp-sharing-button__icon--solidcircle">
            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M12 23.5c6.35 0 11.5-5.15 11.5-11.5S18.35.5 12 .5.5 5.65.5 12 5.65 23.5 12 23.5zM2.505 11.053c-.31.118-.505.738-.505.738s.203.62.513.737l3.636 1.355 1.417 4.557a.787.787 0 0 0 1.25.375l2.115-1.72a.29.29 0 0 1 .353-.01L15.1 19.85a.786.786 0 0 0 .746.095.786.786 0 0 0 .487-.573l2.793-13.426a.787.787 0 0 0-1.054-.893l-15.568 6z" fill-rule="evenodd"/></svg>
            </div>
        </div>
        </a>
    </span>
    """

    # Define button html
    my_share_button = f"""
    <style>
    {my_css}
    </style>
    {my_html}
    """

    # Render
    html(my_share_button)

    return


import streamlit as st
from glob import glob
import os
import pandas as pd

try:
    from .render import render_file
except:
    from render import render_file

def on_toc_table_click():
    st.session_state.toc = False
    return

def on_previous_click():
    """
    Updates the values of update_from_selectbox and update_from_button.
    Updates the page number to +1, or 0 if the last page was reached.
    """
    st.session_state.page_number = (st.session_state.page_number - 1) % st.session_state.total_files
    return

def on_next_click():
    """
    Updates the values of update_from_selectbox and update_from_button.
    Updates the page number to +1, or 0 if the last page was reached.
    """
    st.session_state.page_number = (st.session_state.page_number + 1) % st.session_state.total_files
    return

def on_refresh_click():
    return

def create_buttons(caption_text, 
                    button_previous, 
                    button_next, 
                    button_refresh,
                    key=""):
    """
    Function to create the navigation buttons.
    Only applies if more than 1 file is to be rendered.
    """
    # Skip if there is only one file
    if st.session_state.total_files <= 1:
        return
    # We have at least 2 files, so render the buttons
    st.caption(caption_text)
    p = len(button_previous)
    n = len(button_next)
    r = len(button_refresh)
    b = max(p,n,r)
    # Dinamic resizing based on length of the button names
    # Would be even better if layout type was known
    c1, c2, c3, c4 = st.columns([b, b, b, max(b, 20-3*b)])
    if st.session_state.page_number!=0:
        c1.button(button_previous, 
                    help="Previous page", on_click=on_previous_click, key="previous_button_top"+key)
    if st.session_state.page_number!=st.session_state.total_files-1:
        c2.button(button_next, 
                    help="Next page", on_click=on_next_click, key="next_button_top"+key)
    c3.button(button_refresh, 
            help="Refresh current page", on_click=on_refresh_click, key="switch_button_top"+key)
    return

def get_all_files(path):
    """
    Returns a list of all files (python, markdown) in the given path, 
    considering recursively all subfolders.
    The path can be a single file.
    It does not considers folders.
    Excludes files and folders starting with WIP (work in progress).
    It stores the total number of files (pages) in the session_state.
    """
    if path.endswith(".py"):
        py_files = [ path, ]
    else:
        py_files = glob(f"{path}/**/*.py", recursive=True)
    if path.endswith(".md"):
        md_files = [ path, ]
    else:
        md_files = glob(f"{path}/**/*.md", recursive=True)
    all_files = [_ for _ in sorted(py_files + md_files) if "/WIP" not in _]
    st.session_state.total_files = len(all_files)
    return all_files

def is_file(label):
    """
    Checks if the given label is a file.
    """
    return label.endswith(".py") or label.endswith(".md")

def get_options_at_path(user_selected_path: str, file_list: list):
    """
    Returns all the alternative items (folders and files) on the provided path
    """
    # Get the number of folders at the current level
    n_folders = len(user_selected_path.split("/"))
    # Get all the files at level
    subfolder_options = []
    for filepath in file_list:
        subfolder = "/".join(filepath.split("/")[:(n_folders+1)])
        if filepath.startswith(user_selected_path) and subfolder not in subfolder_options:
            subfolder_options.append(subfolder)
    options = [_.split("/")[-1] for _ in subfolder_options]
    return options

def get_selection_box_args_at_path(user_selected_path: str, file_list: list, level: int, selected_option: str=""):
    """
    Search for items: folders, python files or markdown files.
    Only looks for files at the given path.
    It returns a dictionary, with the required keys and values required by the selectbox
    """
    # Get all options at certain level
    options = get_options_at_path(user_selected_path, file_list)
    # Get the current label index
    if selected_option:
        index = options.index(selected_option)
    else:
        index = 0
    # Get all options at certain level
    selection_box_args = {
                        "label": f"Select {book_parts_convention(level)}:",
                        "options": options, 
                        "index": index,
                        "preset_option": options[index],                        
                        "key": f"select_box_{level}",
                        }
    return selection_box_args

def get_selection_boxes_args_from_filepath(file_fullpath: str, base_path: str, file_list: list):
    """
    Creates a list of dictionaries, with the required keys and values required by the selectbox.
    """
    # The relative path are only the folders in the filepath not on the base_path (but not the file itself)
    relative_path = file_fullpath.replace(f"{base_path}/", "")
    # Get the intermediate folders, but not the file itself
    selected_labels = relative_path.split("/")
    # Initialize and populate the list of dictionaries
    selection_boxes_args = []
    working_path = base_path
    for level, label in enumerate(selected_labels):
        # Get the options of the selectbox
        selection_box_args = get_selection_box_args_at_path(working_path, file_list, level, label)
        selection_boxes_args.append(selection_box_args)
        # Increate the path
        working_path = working_path + "/" + label
    return selection_boxes_args

def get_items(path: str):
    """
    # Search for folders, python files or markdown files.
    Only looks at precisely the level of the given path.
    It returns a dictionary, with keys the name to be rendered in the selectbox
    and with values another dictionary, with the type (file/folder) and the fullpath.
    """
    # Get all the files
    items_at_path = glob(f"{path}/*/") + glob(f"{path}/*.md") + glob(f"{path}/*.py")
    # Filter files that start with WIP (work in progress)
    items_at_path = [item for item in items_at_path if ("/WIP" not in item)]
    # Create
    item_dict = {}
    for i, my_item in enumerate(items_at_path):
        if my_item[-3:] in (".py", ".md"):
            item_name = my_item.split("/")[-1]
            render_name = item_name
            item_dict[render_name] = {"type":"file", "path":my_item}
        else:
            item_name = my_item.split("/")[-2]
            render_name = item_name
            item_dict[render_name] = {"type":"folder", "path":my_item}
    return item_dict



import streamlit as st
import streamlit_book as stb
import time
import random

def main():
    # Set wide display
    st.set_page_config(layout="wide", page_icon="📝", page_title="Activities Demo 0.7.0")

    # Streamlit content
    st.title("Basic streamlit_book behavior")
    st.write("This tests mixing streamlit and streamlit_book libraries. Notice that the streamlit_book library is imported as stb but no multipage capability is enabled!!! Similar examples can be constructed for other activities.")

    # Default
    st.header("Question with minimal arguments")
    c1, c2 = st.columns([5,5])
    with c1:
        st.code("""
stb.true_or_false("Are you a robot?", False)
            """)
    with c2:
        stb.true_or_false("Are you a robot?", False)

    # Default
    st.header("Question with all optional arguments")
    c1, c2 = st.columns([5,4])
    with c1:
        st.code("""
stb.true_or_false("Are you a cyborg?", False, 
                  success='Pfiuuuuu!!!', 
                  error='RoBoTs NoT WeLcOmE to tHiS aPp', 
                  button='Check MY answer')
            """)
    with c2:
        stb.true_or_false("Are you a cyborg?", False, 
                            success='Pfiuuuuu!!!', 
                            error='RoBoTs NoT WeLcOmE to tHiS aPp', 
                            button='Check MY answer')

    # Custom question
    st.header("Question with custom behavior")
    c1, c2 = st.columns([4,3])
    with c1:
        st.code("""
checked_answer, correct_answer = stb.true_or_false("Are you a cyborg robot?", 
                                                   False, 
                                                   success="",
                                                   error="",
                                                   button='Check THIS answer')
if checked_answer:
    if correct_answer:
        st.info("Welcome fellow human!")            
        st.balloons()
    else:
        N = 10.0
        pb = st.progress(0.0)
        ph = st.empty()
        robot_message = ""
        for i in range(1, int(N+1)):
            pb.progress(i/N)
            robot_message += random.choice(["Bip ", "bip ", "Bop ", "bop "])
            ph.code(robot_message)
            time.sleep(.5)
else:
    st.write("You need to check the answer")
            """)
    with c2:
        checked_answer, correct_answer = stb.true_or_false("Are you a cyborg robot?", 
                                                            False, 
                                                            success="",
                                                            error="",
                                                            button='Check THIS answer')
        if checked_answer:
            if correct_answer:
                st.info("Welcome fellow human!")            
                st.balloons()
            else:
                N = 10.0
                pb = st.progress(0.0)
                ph = st.empty()
                robot_message = ""
                for i in range(1, int(N+1)):
                    pb.progress(i/N)
                    robot_message += random.choice(["BIP ", "bip ", "BOP ", "bop "])
                    ph.code(robot_message)
                    time.sleep(.5)
        else:
            st.write("You need to check the answer")
    

if __name__ == "__main__":
    main()


# Library to configure this setup file
import pathlib
from setuptools import find_packages, setup

# VERSION - is a variable that is defined in the __init__.py file
VERSION = "0.7.0"

HERE = pathlib.Path(__file__).parent
PACKAGE_NAME = 'streamlit_book' #Debe coincidir con el nombre de la carpeta 
AUTHOR = 'Sebastián Flores Benner'
AUTHOR_EMAIL = 'sebastiandres@gmail.com'
URL = 'https://github.com/sebastiandres/streamlit_book'

LICENSE = 'MIT' #Tipo de licencia
DESCRIPTION = 'A streamlit companion library to create a interactive reader for the content on a given folder.'
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8')
LONG_DESC_TYPE = "text/markdown"
PROJECT_URLS = {
                "Documentation": "https://streamlit-book.readthedocs.io/",
                'Source': 'https://github.com/sebastiandres/streamlit_book/',
                'Tracker': 'https://github.com/sebastiandres/streamlit_book/issues',
                }
# Libraries required by the package
INSTALL_REQUIRES = ['streamlit', 'streamlit-option-menu']

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    packages=find_packages(),
    include_package_data=True,    
    url=URL,
    project_urls=PROJECT_URLS,
    license=LICENSE,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    setup_requires=INSTALL_REQUIRES,
    install_requires=INSTALL_REQUIRES,
    )



import streamlit as st

try:
    from keywords import *
except:
    from .keywords import *

def text_input_from_lines(lines):
    """Parses a list of lines into a dictionary with the parsed values.

    :param lines: list of lines
    :type lines: list
    :return: Returns a dict of the parsed values for the text input quizz type.
    :rtype: dict
    """    # Define default feedback messages 
    quizz = {
            SUCCESS:DEFAULT_SUCCESS_MESSAGE, 
            ERROR:DEFAULT_ERROR_MESSAGE, 
            BUTTON:DEFAULT_BUTTON_MESSAGE,
            "question":"",
            "answer_check":{},
            }
    # Iterate through lines and process each line accordingly
    for i, line in enumerate(lines):
        if i==0:
            if line.startswith(TEXT_INPUT_KEYWORD):
                continue
            else:
                raise st.error(f"Error in the format.")
        elif line.startswith(EXACT_TEXT):
            msg = line[len(EXACT_TEXT):].strip()
            quizz["answer_check"][EXACT_TEXT] = msg
        elif line.startswith(CONTAINS_TEXT):
            msg = line[len(CONTAINS_TEXT):].strip()
            quizz["answer_check"][CONTAINS_TEXT] = msg
        elif line.startswith(STARTS_WITH):
            msg = line[len(STARTS_WITH):].strip()
            quizz["answer_check"][STARTS_WITH] = msg
        elif line.startswith(ENDS_WITH):
            msg = line[len(ENDS_WITH):].strip()
            quizz["answer_check"][ENDS_WITH] = msg
        elif line.startswith(BUTTON):
            msg = line[len(BUTTON):].strip()
            quizz[BUTTON] = msg
        elif line.startswith(SUCCESS):
            msg = line[len(SUCCESS):].strip()
            quizz[SUCCESS] = msg  
        elif line.startswith(ERROR):
            msg = line[len(ERROR):].strip()
            quizz[ERROR] = msg
        else:
            # Add the line to question text
            quizz["question"] += line

    # Create the quiz
    user_answer = st.text_area(quizz["question"])
    if st.button(quizz[BUTTON]):
        if passes_all_tests(user_answer, quizz):
            st.success(quizz[SUCCESS])
        else:
            st.error(quizz[ERROR])
    return

def passes_all_tests(user_answer, quizz):
    """
    """
    check_list = []
    if EXACT_TEXT in quizz["answer_check"]:
        check_list.append(user_answer.strip()==quizz["answer_check"][EXACT_TEXT].strip())
    if CONTAINS_TEXT in quizz["answer_check"]:
        check_list.append(quizz["answer_check"][CONTAINS_TEXT] in user_answer)
    if ENDS_WITH in quizz["answer_check"]:
        check_list.append(user_answer.endswith(quizz["answer_check"][ENDS_WITH]))
    if STARTS_WITH in quizz["answer_check"]:
        check_list.append(user_answer.startswith(quizz["answer_check"][STARTS_WITH]))
    return all(check_list)


import streamlit as st

try:
    from keywords import check_keyword
    from keywords import MULTIPLE_CHOICE_KEYWORD, MULTIPLE_CHOICE_FALSE, MULTIPLE_CHOICE_TRUE
    from keywords import BUTTON, SUCCESS, ERROR, DEFAULT_SUCCESS_MESSAGE, DEFAULT_ERROR_MESSAGE, DEFAULT_BUTTON_MESSAGE
    from answers import save_answer
except:
    from .keywords import check_keyword
    from .keywords import MULTIPLE_CHOICE_KEYWORD, MULTIPLE_CHOICE_FALSE, MULTIPLE_CHOICE_TRUE
    from .keywords import BUTTON, SUCCESS, ERROR, DEFAULT_SUCCESS_MESSAGE, DEFAULT_ERROR_MESSAGE, DEFAULT_BUTTON_MESSAGE
    from .answers import save_answer


def multiple_choice(question, options_dict,  
                    success=DEFAULT_SUCCESS_MESSAGE, 
                    error=DEFAULT_ERROR_MESSAGE, 
                    button=DEFAULT_BUTTON_MESSAGE):
    """Render a multiple choice question from the given parameters.

    :param question: question to be displayed before the multiple-choice options
    :type question: str
    :param options_dict: dictionary of options to be displayed, with the option text as key and the boolean answer as value
    :type options: dict
    :param answer: index (starting at 0) of the expected answer to the question
    :type answer: int
    :param success: message to be displayed when the user answers correctly
    :type success: str, optional
    :param error: message to be displayed when the user answers incorrectly
    :type error: str, optional
    :param button: message to be displayed on the button that checks the answer
    :type button: str, optional
    :return: tuple of booleans with button press status and correctness of answer
    :rtype: tuple of bool
    """
    cb_list = []
    if len(question)==0:
        st.error("Please provide a question")
        return None, None
    elif len(options_dict)<2:
        st.error("Must have at least 2 options")
        return None, None
    else:
        # Write the question
        st.markdown(question)
        # Write the options and append the user answers into the list cb_list
        for option, answer in options_dict.items():
            key = (question + option + str(answer)).lower().replace(" ", "_")
            # Post the option with no option
            cb = st.checkbox(option, value=False, key=key)
            cb_list.append(cb)
        # Check if the correct checkboxes are checked
        key = ("multiple-choice:" + question + "".join(options_dict.keys())).lower().replace(" ", "_")
        if st.button(button, key=key):
            # Check answers
            if all(u==a for u, a in zip(cb_list, options_dict.values())):
                if success:
                    st.success(success)
                is_correct = True
            else:
                if error:
                    st.error(error)
                is_correct = False
            # Save the answers, if required
            if "save_answers" in st.session_state and st.session_state.save_answers:                
                user_answer = [list(options_dict.keys())[i] for i, cb in enumerate(cb_list) if cb]
                correct_answer = [key for key, val in options_dict.items() if val]
                save_answer(question, is_correct=is_correct, user_answer=user_answer, correct_answer=correct_answer)
            # Return if button is pressed and answer evaluation
            return True, is_correct
        else:
            # Return if button is not pressed
            return False, None


def multiple_choice_parser(lines):
    """Parses a list of lines into a dictionary with the parsed values.

    :param lines: list of lines
    :type lines: list
    :return: parsed values for the multiple choice quizz type.
    :rtype: dict
    """
    # Define default feedback messages 
    parse_dict = {
                    "question":"",
                    "options_dict":{},
                  }
    # Iterate through lines and process each line accordingly
    for i, line in enumerate(lines):
        if i==0:
            if check_keyword(line, MULTIPLE_CHOICE_KEYWORD):
                continue
        elif line.startswith(MULTIPLE_CHOICE_TRUE):
            option_text = line[len(MULTIPLE_CHOICE_TRUE):].strip()
            parse_dict["options_dict"][option_text] = True
        elif line.startswith(MULTIPLE_CHOICE_FALSE):
            option_text = line[len(MULTIPLE_CHOICE_FALSE):].strip()
            parse_dict["options_dict"][option_text] = False
        elif line.startswith(BUTTON):
            msg = line[len(BUTTON):].strip()
            parse_dict["button"] = msg  
        elif line.startswith(SUCCESS):
            msg = line[len(SUCCESS):].strip()
            parse_dict["success"] = msg  
        elif line.startswith(ERROR):
            msg = line[len(ERROR):].strip()
            parse_dict["error"] = msg
        else:
            # Put the rest of the lines into header as markdown
            parse_dict["question"] += line + "\n"
    return parse_dict


def multiple_choice_from_lines(lines):
    """Renders a multiple choice question from a list of lines.

    :param lines: list of lines
    :type lines: list
    :return: None
    """
    parse_dict = multiple_choice_parser(lines)
    multiple_choice(**parse_dict)
    return